@@ -9,6 +9,7 @@
  * license.
  */
 
+#include <stdlib.h>
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -18,9 +19,10 @@
 
 
 enum {
-    RENDER_PREVIEW,
     START_SCAN,
     STOP_SCAN,
+    ROTATE_LEFT,
+    ROTATE_RIGHT,
     SAVE,
     PRINT,
     QUIT,
@@ -34,15 +36,16 @@ struct SimpleScanPrivate
     GConfClient *client;
 
     GtkWidget *window;
-    GtkWidget *scan_label;
+    GtkWidget *scan_button_label, *continuous_scan_button_label, *page_label;
     GtkWidget *actions_box;
     GtkWidget *device_combo, *mode_combo;
     GtkTreeModel *device_model, *mode_model;
+    GtkWidget *replace_pages_check;
     GtkWidget *preview_area;
+    GtkWidget *zoom_scale;
 
     gchar *default_file_name;
     gboolean scanning;
-    Orientation orientation;
 };
 
 G_DEFINE_TYPE (SimpleScan, ui, G_TYPE_OBJECT);
@@ -67,21 +70,6 @@ find_scan_device (SimpleScan *ui, const char *device, GtkTreeIter *iter)
 }
 
 
-static gchar *
-get_selected_device (SimpleScan *ui)
-{
-    GtkTreeIter iter;
-
-    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter)) {
-        gchar *device;
-        gtk_tree_model_get (ui->priv->device_model, &iter, 0, &device, -1);
-        return device;
-    }
-
-    return NULL;
-}
-
-
 void
 ui_set_default_file_name (SimpleScan *ui, const gchar *default_file_name)
 {
@@ -121,6 +109,21 @@ ui_add_scan_device (SimpleScan *ui, const gchar *device, const gchar *label)
 }
 
 
+gchar *
+ui_get_selected_device (SimpleScan *ui)
+{
+    GtkTreeIter iter;
+
+    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter)) {
+        gchar *device;
+        gtk_tree_model_get (ui->priv->device_model, &iter, 0, &device, -1);
+        return device;
+    }
+
+    return NULL;
+}
+
+
 void
 ui_set_selected_device (SimpleScan *ui, const gchar *device)
 {
@@ -172,22 +175,10 @@ get_document_hint (SimpleScan *ui)
 }
 
 
-G_MODULE_EXPORT
-gboolean
-preview_area_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, SimpleScan *ui)
+static gboolean
+get_replace_pages (SimpleScan *ui)
 {
-    cairo_t *context;
-    double width, height;
-    
-    context = gdk_cairo_create (widget->window);
-    
-    width = widget->allocation.width;
-    height = widget->allocation.height;
-    g_signal_emit (G_OBJECT (ui), signals[RENDER_PREVIEW], 0, context, width, height);
-
-    cairo_destroy (context);
-
-    return FALSE;
+    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check));
 }
 
 
@@ -200,10 +191,11 @@ scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
     } else {
         gchar *device, *mode;
 
-        device = get_selected_device (ui);
+        device = ui_get_selected_device (ui);
         if (device) {
             mode = get_document_hint (ui);
-            g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode);
+            g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode,
+                           FALSE, get_replace_pages (ui));
             g_free (device);
             g_free (mode);
         }
@@ -213,12 +205,38 @@ scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 
 G_MODULE_EXPORT
 void
-rotate_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+continuous_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    ui->priv->orientation++;
-    if (ui->priv->orientation > RIGHT_TO_LEFT)
-        ui->priv->orientation = TOP_TO_BOTTOM;
-    ui_redraw_preview (ui);
+    if (ui->priv->scanning) {
+        g_signal_emit (G_OBJECT (ui), signals[STOP_SCAN], 0);
+    } else {
+        gchar *device, *mode;
+
+        device = ui_get_selected_device (ui);
+        if (device) {
+            mode = get_document_hint (ui);
+            g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode,
+                           TRUE, get_replace_pages (ui));
+            g_free (device);
+            g_free (mode);
+        }
+    }
+}
+
+
+G_MODULE_EXPORT
+void
+rotate_left_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    g_signal_emit (G_OBJECT (ui), signals[ROTATE_LEFT], 0);
+}
+
+
+G_MODULE_EXPORT
+void
+rotate_right_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    g_signal_emit (G_OBJECT (ui), signals[ROTATE_RIGHT], 0);
 }
 
 
@@ -408,14 +426,52 @@ save_device_cache (SimpleScan *ui)
 
 
 G_MODULE_EXPORT
-gboolean
-window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
+void
+about_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    const gchar *authors[] = { "Robert Ancell <robert.ancell@canonical.com>", NULL };
+
+    /* The license this software is under (GPL3+) */
+    const char *license = _("This program is free software: you can redistribute it and/or modify\n"
+                            "it under the terms of the GNU General Public License as published by\n"
+                            "the Free Software Foundation, either version 3 of the License, or\n"
+                            "(at your option) any later version.\n"
+                            "\n"
+                            "This program is distributed in the hope that it will be useful,\n"
+                            "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+                            "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+                            "GNU General Public License for more details.\n"
+                            "\n"
+                            "You should have received a copy of the GNU General Public License\n"
+                            "along with this program.  If not, see <http://www.gnu.org/licenses/>.");
+
+    /* Title of about dialog */
+    const char *title = _("About Simple Scan");
+
+    /* Description of program */
+    const char *description = _("Simple document scanning tool");
+
+    gtk_show_about_dialog (GTK_WINDOW (ui->priv->window),
+                           "title", title,
+                           "program-name", "Simple Scan",
+                           "version", VERSION,
+                           "comments", description,
+                           "authors", authors,
+                           "website", "https://launchpad.net/simple-scan",
+                           "copyright", "Copyright (C) 2009 Canonical Ltd.",
+                           "license", license,
+                           NULL);
+}
+
+
+static void
+quit (SimpleScan *ui)
 {
     char *device, *document_type;
 
     save_device_cache (ui);
 
-    device = get_selected_device (ui);
+    device = ui_get_selected_device (ui);
     if (device) {
         gconf_client_set_string(ui->priv->client, "/apps/simple-scan/selected_device", device, NULL);
         g_free (device);
@@ -425,36 +481,64 @@ window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
     gconf_client_set_string(ui->priv->client, "/apps/simple-scan/document_type", document_type, NULL);
     g_free (document_type);
 
+    gconf_client_set_bool(ui->priv->client, "/apps/simple-scan/replace_pages",
+                          get_replace_pages (ui), NULL);
+
     g_signal_emit (G_OBJECT (ui), signals[QUIT], 0);
+}
+
+
+G_MODULE_EXPORT
+void
+quit_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    quit (ui);
+}
+
+
+G_MODULE_EXPORT
+gboolean
+window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
+{
+    quit (ui);
     return TRUE;
 }
 
 
-static gboolean
+static void
 ui_load (SimpleScan *ui)
 {
     GtkBuilder *builder;
     GError *error = NULL;
     GtkCellRenderer *renderer;
     gchar *device, *document_type;
+    gboolean replace_pages;
 
     builder = gtk_builder_new ();
     gtk_builder_add_from_file (builder, UI_DIR "simple-scan.ui", &error);
     if (error) {
-        // FIXME: Error dialog
         g_critical ("Unable to load UI: %s\n", error->message);
-        return FALSE;
+        ui_show_error (ui,
+                       /* Title of dialog when cannot load required files */
+                       _("Files missing"),
+                       /* Description in dialog when cannot load required files */
+                       _("Please check your installation"));
+        exit (1);
     }
     gtk_builder_connect_signals (builder, ui);
 
     ui->priv->window = GTK_WIDGET (gtk_builder_get_object (builder, "simple_scan_window"));
-    ui->priv->scan_label = GTK_WIDGET (gtk_builder_get_object (builder, "scan_label"));
+    ui->priv->scan_button_label = GTK_WIDGET (gtk_builder_get_object (builder, "scan_button_label"));
+    ui->priv->continuous_scan_button_label = GTK_WIDGET (gtk_builder_get_object (builder, "continuous_scan_button_label"));
+    ui->priv->page_label = GTK_WIDGET (gtk_builder_get_object (builder, "page_label"));
     ui->priv->actions_box = GTK_WIDGET (gtk_builder_get_object (builder, "actions_box"));
     ui->priv->device_combo = GTK_WIDGET (gtk_builder_get_object (builder, "device_combo"));
     ui->priv->device_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->device_combo));
     ui->priv->mode_combo = GTK_WIDGET (gtk_builder_get_object (builder, "mode_combo"));
     ui->priv->mode_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->mode_combo));
+    ui->priv->replace_pages_check = GTK_WIDGET (gtk_builder_get_object (builder, "replace_pages_check"));
     ui->priv->preview_area = GTK_WIDGET (gtk_builder_get_object (builder, "preview_area"));
+    ui->priv->zoom_scale = GTK_WIDGET (gtk_builder_get_object (builder, "zoom_scale"));
 
     renderer = gtk_cell_renderer_text_new();
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->device_combo), renderer, TRUE);
@@ -481,7 +565,11 @@ ui_load (SimpleScan *ui)
         g_free (document_type);
     }
 
-    return TRUE;
+    replace_pages = gconf_client_get_bool (ui->priv->client, "/apps/simple-scan/replace_pages", &error);
+    if (error)
+        g_error_free (error);
+    else
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check), replace_pages);                                  
 }
 
 
@@ -492,18 +580,40 @@ ui_new ()
 }
 
 
+GtkWidget *
+ui_get_preview_widget (SimpleScan *ui)
+{
+    return ui->priv->preview_area;
+}
+
+
+void
+ui_set_zoom_adjustment (SimpleScan *ui, GtkAdjustment *adjustment)
+{
+    gtk_range_set_adjustment (GTK_RANGE (ui->priv->zoom_scale), adjustment);
+}
+
+
 void
 ui_set_scanning (SimpleScan *ui, gboolean scanning)
 {
     ui->priv->scanning = scanning;
-    if (ui->priv->scanning)
-        gtk_label_set_label (GTK_LABEL (ui->priv->scan_label),
+    if (ui->priv->scanning) {
+        gtk_label_set_label (GTK_LABEL (ui->priv->scan_button_label),
                              /* Label on cancel scan button */
                              _("_Cancel"));
-    else
-        gtk_label_set_label (GTK_LABEL (ui->priv->scan_label),
+        gtk_label_set_label (GTK_LABEL (ui->priv->continuous_scan_button_label),
+                             /* Label on cancel scan button */
+                             _("_Cancel"));
+    }
+    else {
+        gtk_label_set_label (GTK_LABEL (ui->priv->scan_button_label),
                              /* Label on scan button */
                              _("_Scan"));
+        gtk_label_set_label (GTK_LABEL (ui->priv->continuous_scan_button_label),
+                             /* Label on continuous scan button */
+                             _("_Continuous Scan"));
+    }
 }
 
 
@@ -514,19 +624,6 @@ ui_set_have_scan (SimpleScan *ui, gboolean have_scan)
 }
 
 
-Orientation ui_get_orientation (SimpleScan *ui)
-{
-    return ui->priv->orientation;
-}
-
-
-void
-ui_redraw_preview (SimpleScan *ui)
-{
-    gtk_widget_queue_draw (ui->priv->preview_area);    
-}
-
-
 void
 ui_show_error (SimpleScan *ui, const gchar *error_title, const gchar *error_text)
 {
@@ -548,34 +645,28 @@ ui_show_error (SimpleScan *ui, const gchar *error_title, const gchar *error_text
 void
 ui_start (SimpleScan *ui)
 {
-    if (gtk_tree_model_iter_n_children (ui->priv->device_model, NULL) == 0)
-        ui_show_error (ui,
-                       /* Warning displayed when no scanners are detected */
-                       _("No scanners detected"),
-                       /* Hint to user on why there are no scanners detected */
-                       _("Please check your scanner is connected and powered on"));
+    gtk_widget_show (ui->priv->window);
 }
 
 
 /* Generated with glib-genmarshal */
 static void
-g_cclosure_user_marshal_VOID__POINTER_DOUBLE_DOUBLE (GClosure     *closure,
-                                                     GValue       *return_value G_GNUC_UNUSED,
-                                                     guint         n_param_values,
-                                                     const GValue *param_values,
-                                                     gpointer      invocation_hint G_GNUC_UNUSED,
-                                                     gpointer      marshal_data)
+g_cclosure_user_marshal_VOID__POINTER_POINTER (GClosure     *closure,
+                                               GValue       *return_value G_GNUC_UNUSED,
+                                               guint         n_param_values,
+                                               const GValue *param_values,
+                                               gpointer      invocation_hint G_GNUC_UNUSED,
+                                               gpointer      marshal_data)
 {
-    typedef void (*GMarshalFunc_VOID__POINTER_DOUBLE_DOUBLE) (gpointer     data1,
-                                                              gpointer     arg_1,
-                                                              gdouble      arg_2,
-                                                              gdouble      arg_3,
-                                                              gpointer     data2);
-    register GMarshalFunc_VOID__POINTER_DOUBLE_DOUBLE callback;
+    typedef void (*GMarshalFunc_VOID__POINTER_POINTER) (gpointer     data1,
+                                                                     gpointer     arg_1,
+                                                                     gpointer     arg_2,
+                                                                     gpointer     data2);
+    register GMarshalFunc_VOID__POINTER_POINTER callback;
     register GCClosure *cc = (GCClosure*) closure;
     register gpointer data1, data2;
     
-    g_return_if_fail (n_param_values == 4);
+    g_return_if_fail (n_param_values == 3);
     
     if (G_CCLOSURE_SWAP_DATA (closure))
     {
@@ -587,34 +678,35 @@ g_cclosure_user_marshal_VOID__POINTER_DOUBLE_DOUBLE (GClosure     *closure,
         data1 = g_value_peek_pointer (param_values + 0);
         data2 = closure->data;
     }
-    callback = (GMarshalFunc_VOID__POINTER_DOUBLE_DOUBLE) (marshal_data ? marshal_data : cc->callback);
+    callback = (GMarshalFunc_VOID__POINTER_POINTER) (marshal_data ? marshal_data : cc->callback);
     
     callback (data1,
               g_value_get_pointer (param_values + 1),
-              g_value_get_double (param_values + 2),
-              g_value_get_double (param_values + 3),
+              g_value_get_pointer (param_values + 2),
               data2);
 }
 
 
 /* Generated with glib-genmarshal */
 void
-g_cclosure_user_marshal_VOID__STRING_STRING (GClosure     *closure,
-                                             GValue       *return_value G_GNUC_UNUSED,
-                                             guint         n_param_values,
-                                             const GValue *param_values,
-                                             gpointer      invocation_hint G_GNUC_UNUSED,
-                                             gpointer      marshal_data)
+g_cclosure_user_marshal_VOID__STRING_STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
+                                                             GValue       *return_value G_GNUC_UNUSED,
+                                                             guint         n_param_values,
+                                                             const GValue *param_values,
+                                                             gpointer      invocation_hint G_GNUC_UNUSED,
+                                                             gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_STRING) (gpointer       data1,
-                                                    gconstpointer  arg_1,
-                                                    gconstpointer  arg_2,
-                                                    gpointer       data2);
-  register GMarshalFunc_VOID__STRING_STRING callback;
+  typedef void (*GMarshalFunc_VOID__STRING_STRING_BOOLEAN_BOOLEAN) (gpointer       data1,
+                                                                    gconstpointer  arg_1,
+                                                                    gconstpointer  arg_2,
+                                                                    gboolean       arg_3,
+                                                                    gboolean       arg_4,
+                                                                    gpointer       data2);
+  register GMarshalFunc_VOID__STRING_STRING_BOOLEAN_BOOLEAN callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
-  g_return_if_fail (n_param_values == 3);
+  g_return_if_fail (n_param_values == 5);
 
   if (G_CCLOSURE_SWAP_DATA (closure))
     {
@@ -626,11 +718,13 @@ g_cclosure_user_marshal_VOID__STRING_STRING (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__STRING_STRING) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__STRING_STRING_BOOLEAN_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
             g_value_get_string (param_values + 1),
             g_value_get_string (param_values + 2),
+            g_value_get_boolean (param_values + 3),
+            g_value_get_boolean (param_values + 4),
             data2);
 }
 
@@ -638,22 +732,14 @@ g_cclosure_user_marshal_VOID__STRING_STRING (GClosure     *closure,
 static void
 ui_class_init (SimpleScanClass *klass)
 {
-    signals[RENDER_PREVIEW] =
-        g_signal_new ("render-preview",
-                      G_TYPE_FROM_CLASS (klass),
-                      G_SIGNAL_RUN_LAST,
-                      G_STRUCT_OFFSET (SimpleScanClass, render_preview),
-                      NULL, NULL,
-                      g_cclosure_user_marshal_VOID__POINTER_DOUBLE_DOUBLE,
-                      G_TYPE_NONE, 3, G_TYPE_POINTER, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
     signals[START_SCAN] =
         g_signal_new ("start-scan",
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SimpleScanClass, start_scan),
                       NULL, NULL,
-                      g_cclosure_user_marshal_VOID__STRING_STRING,
-                      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_STRING);
+                      g_cclosure_user_marshal_VOID__STRING_STRING_BOOLEAN_BOOLEAN,
+                      G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);
     signals[STOP_SCAN] =
         g_signal_new ("stop-scan",
                       G_TYPE_FROM_CLASS (klass),
@@ -662,6 +748,22 @@ ui_class_init (SimpleScanClass *klass)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__VOID,
                       G_TYPE_NONE, 0);
+    signals[ROTATE_LEFT] =
+        g_signal_new ("rotate-left",
+                      G_TYPE_FROM_CLASS (klass),
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (SimpleScanClass, rotate_left),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
+    signals[ROTATE_RIGHT] =
+        g_signal_new ("rotate-right",
+                      G_TYPE_FROM_CLASS (klass),
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (SimpleScanClass, rotate_right),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
     signals[SAVE] =
         g_signal_new ("save",
                       G_TYPE_FROM_CLASS (klass),
@@ -701,7 +803,5 @@ ui_init (SimpleScan *ui)
 
     ui->priv->default_file_name = g_strdup (_("Scanned Document.pdf"));
     ui->priv->scanning = FALSE;
-    ui->priv->orientation = TOP_TO_BOTTOM;
-
     ui_load (ui);
 }
